import typing
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.responses import Response

try:
	from orjson import loads, dumps
except ImportError:
	loads = dumps = None

async def orjson_loads(self) -> typing.Any:
	if not hasattr(self, '_json'):
		body = await self.body()
		self._json = loads(body)
	return self._json

class ORJsonMiddleware(BaseHTTPMiddleware):
	async def dispatch(self, request, call_next):
		response = await call_next(request)
		response.json = orjson_loads
		return response

valid_args = {
	'default', # To serialize a subclass or arbitrary types, specify default as a callable that returns a supported type. default may be a function, lambda, or callable class instance.
	'option', # To specify multiple options, mask them together, e.g., option=orjson.OPT_STRICT_INTEGER | orjson.OPT_NAIVE_UTC
}
class ORJsonResponse(Response):
	'''
	See available dump kwargs here
		https://github.com/ijl/orjson#serialize
	'''
	media_type = "application/json"

	def __init__(self, *args, **kwargs):
		self.render_args = {arg: v for arg,v in kwargs.items() if arg in valid_args}
		base_args = {arg: v for arg,v in kwargs.items() if arg not in valid_args}
		super(ORJsonResponse, self).__init__(*args, **base_args)

	def render(self, content: typing.Any) -> bytes:
		return dumps(content, **self.render_args)



from .starlette_orjson import ORJsonResponse, ORJsonMiddleware
from .starlette_ujson import UJsonResponse, UJsonMiddleware
from .starlette_rapidjson import RapidJsonResponse, RapidJsonMiddleware
from .starlette_simplejson import SimpleJsonResponse, SimpleJsonMiddleware


import typing
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.responses import Response

try:
	from rapidjson import loads, dumps
except ImportError:
	loads = dumps = None

async def rapidjson_loads(self) -> typing.Any:
	print('00000')
	if not hasattr(self, '_json'):
		body = await self.body()
		print(self.json_opts)
		self._json = loads(body, **self.json_opts)
	return self._json

class RapidJsonMiddleware(BaseHTTPMiddleware):
	def __init__(self, app, **kwargs):
		self.app = app
		self.json_opts = kwargs
		print(self.json_opts)
		super(RapidJsonMiddleware, self).__init__(app)

	async def dispatch(self, request, call_next):
		response = await call_next(request)
		response.json = rapidjson_loads
		return response

class RapidJsonResponse(Response):
	'''
	See available dumps kwargs here
		https://python-rapidjson.readthedocs.io/en/latest/dumps.html
	'''
	media_type = "application/json"

	valid_args = {
		'skipkeys', # (bool) – whether invalid dict keys will be skipped
		'ensure_ascii', # (bool) – whether the output should contain only ASCII characters
		'indent', # (int) – indentation width to produce pretty printed JSON
		'default', # (callable) – a function that gets called for objects that can’t otherwise be serialized
		'sort_keys', # (bool) – whether dictionary keys should be sorted alphabetically
		'number_mode', # (int) – enable particular behaviors in handling numbers
		'datetime_mode', # (int) – how should datetime, time and date instances be handled
		'uuid_mode', # (int) – how should UUID instances be handled
		'allow_nan' # (bool) – compatibility flag equivalent to number_mode=NM_NAN
	}

	def __init__(self, *args, **kwargs):
		self.render_args = {arg: v for arg,v in kwargs.items() if arg in valid_args}
		base_args = {arg: v for arg,v in kwargs.items() if arg not in valid_args}
		super(RapidJsonResponse, self).__init__(*args, **base_args)

	def render(self, content: typing.Any) -> bytes:
		return dumps(content, **self.render_args).encode('utf-8')



import typing

from starlette.middleware.base import BaseHTTPMiddleware
from starlette.responses import Response

try:
	from simplejson import loads, dumps
except ImportError:
	loads = dumps = None

async def simplejson_loads(self) -> typing.Any:
	if not hasattr(self, '_json'):
		body = await self.body()
		self._json = loads(body, **self.json_opts)
	return self._json

class SimpleJsonMiddleware(BaseHTTPMiddleware):
	def __init__(self, app, **kwargs):
		self.app = app
		self.json_opts = kwargs
		super(SimpleJsonMiddleware, self).__init__(app)

	async def dispatch(self, request, call_next):
		response = await call_next(request)
		response.json = simplejson_loads
		return response

class SimpleJsonResponse(Response):
	'''
	See available dumps kwargs here
		https://simplejson.readthedocs.io/en/latest/#simplejson.dump
	'''
	media_type = "application/json"
	valid_args = {
		'skipkeys',
		'ensure_ascii',
		'check_circular',
		'allow_nan',
		'cls',
		'indent',
		'separators',
		'encoding',
		'default',
		'use_decimal',
		'namedtuple_as_object',
		'tuple_as_array',
		'bigint_as_string',
		'sort_keys',
		'item_sort_key',
		'for_json',
		'ignore_nan',
		'int_as_string_bitcount',
		'iterable_as_array',
	}

	def __init__(self, *args, **kwargs):
		self.render_args = {arg: v for arg,v in kwargs.items() if arg in valid_args}
		base_args = {arg: v for arg,v in kwargs.items() if arg not in valid_args}
		super(SimpleJsonResponse, self).__init__(*args, **base_args)

	def render(self, content: typing.Any) -> bytes:
		return dumps(content, **self.render_args).encode('utf-8')